import click
import multiprocessing
import os
import psutil
import sys
import datetime
import getpass
import requests
import time

from .detector import run_detection, detect_remote_tools
from .logger import get_logger
from .erp import ERPClient

logger = get_logger()

@click.group()
def cli():
    pass

@cli.command()
@click.option('--duration', default=3, type=float, help='Duration in hours')
def start(duration):
    # Prompt for ERP credentials
    print("Login to IIITA ERP to start session:")
    uid = input("Username (UID): ")
    pwd = getpass.getpass("Password: ")
    batch = input("Batch (e.g. 2025): ")
    payload = {
        "uid": uid,
        "pwd": pwd,
        "batch": batch
    }
    try:
        resp = requests.post("http://localhost:5112/api/student/details", json=payload)
        data = resp.json()
        print(data)
        if data.get("status") != "success":
            print("ERP login failed. Please check your credentials.")
            sys.exit(1)
        print("ERP login successful! Fetching ERP details...")
    except Exception as e:
        print(f"Error connecting to iiita_auth service: {e}")
        sys.exit(1)
    # Immediately detect and log current remote tools
    detected_now = detect_remote_tools()
    for d in detected_now:
        app_name = d.get('tool')
        if not app_name:
            app_name = d.get('details', {}).get('name')
        if not app_name:
            app_name = "unknown"
        logger.info({
            "system_username": getpass.getuser(),
            "erp_username": uid,
            "app_name": app_name,
            "timestamp": datetime.datetime.now().isoformat()
        })
    print("Initial detected applications have been logged.")
    pid_file = 'remote_detector.pid'
    if os.path.exists(pid_file):
        with open(pid_file, 'r') as f:
            pid = int(f.read())
        if psutil.pid_exists(pid):
            logger.info({"event": "status", "message": "Already running", "erp_username": uid})
            print("Already running")
            return
    process = multiprocessing.Process(target=run_detection, args=(duration,))
    process.daemon = True
    process.start()
    with open(pid_file, 'w') as f:
        f.write(str(process.pid))
    logger.info({"event": "start", "pid": process.pid, "erp_username": uid})
    print(f"Started with PID {process.pid}")

    duration_seconds = int(duration * 3600)
    end_time = time.time() + duration_seconds

    print(f"Monitoring for flagged applications for {duration} hours...")

    while time.time() < end_time:
        detected_now = detect_remote_tools()
        for d in detected_now:
            app_name = d.get('tool')
            if not app_name:
                app_name = d.get('details', {}).get('name')
            if not app_name:
                app_name = "unknown"
            logger.info({
                "system_username": getpass.getuser(),
                "erp_username": uid,
                "app_name": app_name,
                "timestamp": datetime.datetime.now().isoformat()
            })
        print("Detection cycle complete. Waiting 1 minute for next check...")
        time.sleep(60)

    print("Monitoring complete.")

@cli.command()
def status():
    pid_file = 'remote_detector.pid'
    if not os.path.exists(pid_file):
        print("Not running")
        return
    with open(pid_file, 'r') as f:
        pid = int(f.read())
    if psutil.pid_exists(pid):
        print(f"Running with PID {pid}")
    else:
        print("Not running")
        os.remove(pid_file)

@cli.command()
@click.option('--duration', default=3, type=float, help='Duration in hours')
def debug(duration):
    run_detection(duration)

if __name__ == '__main__':
    cli() 